@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2023 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -16,15 +16,15 @@ import socket
 import time
 import shutil
 import logging
-# from nss import nss
 import subprocess
+import uuid
 from datetime import datetime, timedelta
 from subprocess import check_output, run, PIPE
 from lib389.passwd import password_generate
 from lib389._mapped_object_lint import DSLint
 from lib389.lint import DSCERTLE0001, DSCERTLE0002
-from lib389.utils import ensure_str, format_cmd_list, DSVersion
-import uuid
+from lib389.utils import ensure_str, format_cmd_list, DSVersion, cert_is_ca
+
 
 KEYBITS = 4096
 CA_NAME = 'Self-Signed-CA'
@@ -476,6 +476,32 @@ only.
         result = ','.join([str(elem) for elem in result])
         return result
 
+    def _openssl_get_csr_sub_alt_names(self, csr_dir, csr_name):
+        cmd = [
+            '/usr/bin/openssl',
+            'req',
+            '-noout',
+            '-text',
+            '-in',
+            '%s/%s'% (csr_dir, csr_name),
+        ]
+        self.log.debug("cmd: %s", format_cmd_list(cmd))
+        try:
+            result = ensure_str(check_output(cmd, stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
+
+        subaltnames = []
+        lines = result.split('\n')
+        for line in lines:
+            if 'DNS:' in line:
+                names = line.split(',')
+                for altname in names:
+                    altname = altname.replace("DNS:", "")
+                    subaltnames.append(altname.replace(",:", "").strip())
+
+        return subaltnames
+
     def _csr_show(self, name):
         csr_dir = self.dirsrv.get_cert_dir()
         result = ""
@@ -539,6 +565,8 @@ only.
             csr.append(result.strip())
             # Use openssl to get the csr subject DN
             csr.append(self._openssl_get_csr_subject(csr_dir, csr_file))
+            # Use openssl to get the csr subject alt host names
+            csr.append(self._openssl_get_csr_sub_alt_names(csr_dir, csr_file))
             # Add csr name, without extension
             csr.append(csr_file.rsplit('.', 1)[0])
             csr_list.append(csr)
@@ -1041,7 +1069,6 @@ only.
         except subprocess.CalledProcessError as e:
             raise ValueError(e.output.decode('utf-8').rstrip())
 
-
     def display_cert_details(self, nickname):
         cmd = [
             '/usr/bin/certutil',
@@ -1059,7 +1086,6 @@ only.
 
         return ensure_str(result)
 
-
     def get_cert_details(self, nickname):
         """Get the trust flags, subject DN, issuer, and expiration date
 
@@ -1134,7 +1160,6 @@ only.
             if self._rsa_cert_is_caclienttrust(cert)
         ]
 
-
     def add_cert(self, nickname, input_file, ca=False):
         """Add server or CA cert
         """
@@ -1143,11 +1168,21 @@ only.
         if not os.path.exists(input_file):
             raise ValueError("The certificate file ({}) does not exist".format(input_file))
 
-        self._assert_not_chain(input_file)
+        pem_file = True
+        if not input_file.lower().endswith(".pem"):
+            pem_file = False
+        else:
+            self._assert_not_chain(input_file)
 
         if ca:
+            # Verify this is a CA cert
+            if not cert_is_ca(input_file):
+                raise ValueError(f"Certificate ({nickname}) is not a CA certificate")
             trust_flags = "CT,,"
         else:
+            # Verify this is a server cert
+            if cert_is_ca(input_file):
+                raise ValueError(f"Certificate ({nickname}) is not a server certificate")
             trust_flags = ",,"
 
         cmd = [
@@ -1157,10 +1192,12 @@ only.
             '-n', nickname,
             '-t', trust_flags,
             '-i', input_file,
-            '-a',
             '-f',
             '%s/%s' % (self._certdb, PWD_TXT),
         ]
+        if pem_file:
+            cmd.append('-a')
+
         self.log.debug("add_cert cmd: %s", format_cmd_list(cmd))
         try:
             check_output(cmd, stderr=subprocess.STDOUT)
@@ -1236,6 +1273,12 @@ only.
         if not os.path.exists(cert_file):
             raise ValueError("The certificate file ({}) does not exist".format(cert_file))
 
+        if not cert_file.lower().endswith(".pem"):
+            # Binary cert, this can not be a bundle
+            self.add_cert(nicknames[0], cert_file, ca=True)
+            log.info(f"Successfully added CA certificate ({nicknames[0]})")
+            return
+
         try:
             with open(cert_file, 'r') as f:
                 ca_count = 0
@@ -1260,6 +1303,13 @@ only.
                                 names_len = len(nicknames)
                                 if names_len > ca_count:
                                     if nicknames[ca_count].lower() == CERT_NAME.lower() or nicknames[ca_count].lower() == CA_NAME.lower():
+                                        # cleanup
+                                        try:
+                                            for tmp_file in ca_files_to_cleanup:
+                                                os.remove(tmp_file)
+                                        except IOError as e:
+                                            # failed to remove tmp cert
+                                            log.debug("Failed to remove tmp cert file: " + str(e))
                                         raise ValueError(f"You may not import a CA with the nickname {CERT_NAME} or {CA_NAME}")
                                     ca_cert_name = nicknames[ca_count]
                                 else:
@@ -1279,21 +1329,75 @@ only.
                                     try:
                                         for tmp_file in ca_files_to_cleanup:
                                             os.remove(tmp_file)
-                                    except IOError:
+                                    except IOError as e:
                                         # failed to remove tmp cert
-                                        log.debug("Failed to remove tmp cert file")
-                                        pass
+                                        log.debug("Failed to remove tmp cert file: " + str(e))
                                     raise ValueError(f"Certificate already exists with the same name ({ca_cert_name})")
 
+                                # Verify this is a CA cert
+                                if not cert_is_ca(ca_file_name):
+                                    raise ValueError(f"Certificate ({ca_cert_name}) is not a CA certificate")
+
                                 # Add this CA certificate
                                 self.add_cert(ca_cert_name, ca_file_name, ca=True)
                                 ca_count += 1
                                 log.info(f"Successfully added CA certificate ({ca_cert_name})")
+
+                    # All done, cleanup
+                    try:
+                        for tmp_file in ca_files_to_cleanup:
+                            os.remove(tmp_file)
+                    except IOError as e:
+                        # failed to remove tmp cert
+                        log.debug("Failed to remove tmp cert file: " + str(e))
                 except IOError as e:
                     # Some failure, remove all the tmp files
                     ca_file.close()
-                    for tmp_file in ca_file_to_cleanup:
-                        os.remove(tmp_file)
+                    try:
+                        for tmp_file in ca_files_to_cleanup:
+                            os.remove(tmp_file)
+                    except IOError as e:
+                        # failed to remove tmp cert
+                        log.debug("Failed to remove tmp cert file: " + str(e))
                     raise e
         except EnvironmentError as e:
             raise e
+
+    def export_cert(self, nickname, output_file=None, der_format=False):
+        """
+        :param nickname: name of certificate
+        :param output_file: name for exported certificate
+        :param der_format: export certificate in DER/binary format
+        :raise ValueError: error
+        """
+        cmd = [
+            '/usr/bin/certutil',
+            '-L',
+            '-d', self._certdb,
+            '-n', nickname,
+            '-f',
+            '%s/%s' % (self._certdb, PWD_TXT),
+        ]
+
+        # Handle args for PEM vs DER options
+        if der_format:
+            cmd.append('-r')
+            if output_file is None:
+                output_file = nickname + ".crt"
+        else:
+            cmd.append('-a')
+            if output_file is None:
+                output_file = nickname + ".pem"
+
+        # Set output file name
+        if not output_file.startswith("/"):
+            # Must be full path, otherwise but it in the cert dir
+            output_file = f"{self._certdb}/{output_file}"
+        cmd.append('-o')
+        cmd.append(output_file)
+
+        self.log.debug("export_cert cmd: %s", format_cmd_list(cmd))
+        try:
+            check_output(cmd, stderr=subprocess.STDOUT)
+        except subprocess.CalledProcessError as e:
+            raise ValueError(e.output.decode('utf-8').rstrip())
